package com.test;


import static org.testng.Assert.assertEquals;
import static org.testng.Assert.fail;

import java.util.concurrent.TimeUnit;

import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseConfiguration;
import org.drools.KnowledgeBaseFactory;
import org.drools.audit.WorkingMemoryConsoleLogger;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.conf.EventProcessingOption;
import org.drools.io.ResourceFactory;
import org.drools.runtime.KnowledgeSessionConfiguration;
import org.drools.runtime.StatefulKnowledgeSession;
import org.drools.runtime.conf.ClockTypeOption;
import org.drools.time.SessionPseudoClock;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;


public class TestDroolsEventsIssue {
	
	private KnowledgeBaseConfiguration kbCfg;
	private KnowledgeSessionConfiguration sesCfg;
	
	@BeforeMethod
	public void beforeMethod() throws Exception {
		kbCfg = KnowledgeBaseFactory.newKnowledgeBaseConfiguration();
		kbCfg.setOption( EventProcessingOption.STREAM );
		sesCfg = KnowledgeBaseFactory.newKnowledgeSessionConfiguration();
		sesCfg.setOption( ClockTypeOption.get( "pseudo" ) );
	}
	
	
	@Test
	public void testEventDeclarationInDifferentResource() throws Exception {
		KnowledgeBase kb = getKnowledgeBase(kbCfg, "event_declare.drl", "event_rule.drl");
		final StatefulKnowledgeSession ses = kb.newStatefulKnowledgeSession(sesCfg, null);
		new WorkingMemoryConsoleLogger(ses);
		SessionPseudoClock clock = ses.getSessionClock();
		
		
		ses.insert(new Event1(1));
		ses.fireAllRules();
		clock.advanceTime(2, TimeUnit.SECONDS);
		ses.fireAllRules();
		assertEquals(ses.getFactCount(), 0L);
		
		ses.dispose();
	}
	
	@Test
	public void testEventDeclarationInSameResource() throws Exception {
		KnowledgeBase kb = getKnowledgeBase(kbCfg, "declare_and_rule.drl");
		final StatefulKnowledgeSession ses = kb.newStatefulKnowledgeSession(sesCfg, null);
		new WorkingMemoryConsoleLogger(ses);
		SessionPseudoClock clock = ses.getSessionClock();
				
		ses.insert(new Event1(1));
		ses.fireAllRules();
		clock.advanceTime(2, TimeUnit.SECONDS);
		ses.fireAllRules();
		assertEquals(ses.getFactCount(), 0L);
		
		ses.dispose();
	}
		
	KnowledgeBase getKnowledgeBase(String... resources) {
		return getKnowledgeBase(KnowledgeBaseFactory.newKnowledgeBaseConfiguration(), resources);
	}
	
	KnowledgeBase getKnowledgeBase(KnowledgeBaseConfiguration config, String... resources) {
		KnowledgeBase kb = KnowledgeBaseFactory.newKnowledgeBase(config);
		KnowledgeBuilder builder = KnowledgeBuilderFactory.newKnowledgeBuilder();
		for (String res : resources) {
			builder.add(ResourceFactory.newClassPathResource(res), ResourceType.DRL);
		}
		
		if(!builder.getErrors().isEmpty()) {
			fail(builder.getErrors().toString());
		}
		kb.addKnowledgePackages(builder.getKnowledgePackages());
		return kb;
	}
}
